/*
 * Copyright (C) 2003-2004 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "common/fscapi.h"
#include "common/Log.h"

#include "spdm/common/DeviceManagerFactory.h"
#include "spdm/win32/SyncMLDMManager.h"


#ifdef _WIN32_WCE
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLine, int nShowCmd ) {
#else
int main(int argc, char* argv[]) {
#endif


    #ifdef DEBUG
        Log::debugEnabled = TRUE;
    #else
        Log::debugEnabled = FALSE;
    #endif

    int ret = 0;
    wchar_t sessionId [256]; 
    wchar_t output    [256];
    DeviceManager* deviceManager = NULL;
    DeviceManagerFactory factory;

    wsprintf(sessionId, TEXT("1234")); // dummy session id

    if (argc == 2) {	  
        if (strcmp(argv[1], "-h") == 0 || strcmp(argv[1], "--help") == 0) {
            printf ("\n\tusage: dmclient.exe <sessionId> \n\t sessionId is optional. If omitted dummy sessionId \"1234\" is used.\n\n");
            goto finally;
        }

        MultiByteToWideChar (CP_ACP, 0, argv[1], strlen(argv[1]) + 1, sessionId, strlen(argv[1]) + 1);
	    sessionId[strlen(argv[1])] = 0;       
    }
    
    factory = DeviceManagerFactory();

    //
    // Configurable DMAccount. If it's not setted, default is the first found
    //

    deviceManager = factory.getDeviceManager();
    if (deviceManager != NULL) {
         
        //
        // parameters are: sessionId to use and if isClientNotified(TRUE) or client start communication(FAlSE)
        //
        ret = deviceManager->startSession(sessionId, FALSE);
        
        if (ret != 0) {
            switch (ret) {
                case ERR_PROTOCOL_ERROR:
                    LOG.error(TEXT("Protocol error"));
                    break;

                case ERR_AUTH_NOT_AUTHORIZED:
                    LOG.error(TEXT("Not authorized"));
                    break;
                case ERR_AUTH_REQUIRED:
                    LOG.error(TEXT("Not authorized"));
                    break;

                case ERR_AUTH_EXPIRED:
                    LOG.error(TEXT("Account expired; required payment"));
                    break;

                case ERR_SRV_FAULT:
                    LOG.error(TEXT("Server error"));
                    break;

                case ERR_NOT_FOUND:                
                    LOG.error(TEXT("NotFound"));
                    break;
                default:    
                    break;
            }
            wsprintf(output, TEXT("Result Code: %i"), ret);
            LOG.info(output);
            goto finally;
        }    
        else {
        wsprintf(output, TEXT("DM session complete. Result Code: %i"), ret);
        LOG.info(output);
        }

    }
    else
        LOG.error(TEXT("From client: DeviceManager is NULL"));


finally:  

    if (deviceManager != NULL)
        delete deviceManager;

    return ret;


 }

