/*
 * Copyright (C) 2003-2004 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//
// @author Marco Magistrali
// @version $Id: SyncMLDMManager.cpp,v 1.0 2004/04/27 11:21:00 magi Exp $
//



#include "spds/common/Utils.h"
#include "spdm/common/Utils.h"
#include "spdm/win32/SyncMLDMManager.h"
#include "spdm/win32/Win32ManagementNode.h"
#include "spdm/win32/Win32ManagementTree.h"
#include "spdm/win32/DevInfo.h"
#include "spdm/win32/SyncMLDMConfig.h"
#include "common/Log.h"
#include "spdm/common/md5.h"

#include "spdm/common/wbxml.h"
#include "spdm/common/wbxml_conv.h"


/*
 * Constructor that creates itself a DMConfig and manage a DM session id.
 * If it's NULL then it's created internally.
 */
SyncMLDMManager::SyncMLDMManager() : DeviceManager() {
    LOG.debug(TEXT("COnstructor DMManager"));    

    SyncMLDMManager(NULL);

}

SyncMLDMManager::SyncMLDMManager(wchar_t* accNodeName) {
    wchar_t context [DIM_CONTEXT];   
    
    //init lastErrorCode
    lastErrorCode =         0;
    devInfo =               NULL;
    devDetail =             NULL;
    syncMLDMConfig =        NULL;
    syncMLDMBuilder =       NULL;
    transportAgent =        NULL;
    syncMLDMProcessor =     NULL;
    

    //
    // populate syncMLDMConfig object (with DMAcc and Con)
    //
    
    wsprintf(context, TEXT("%s"), DM_ROOT_CONTEXT);
    ManagementNode* node = getManagementNode(context);  // creted with new operator
    
    syncMLDMConfig = new SyncMLDMConfig(node, accNodeName);
    delete node; node = NULL;
    
    if (lastErrorCode != 0) {
        LOG.info(TEXT("syncMLDMConfig NULL"));
        goto finally;
    }

     //
    // populate DevInfo object 
    //

    wsprintf(context, TEXT("%s/%s"), DM_ROOT_CONTEXT, DM_DEVINFO_CONTEXT);
    node = getManagementNode(context);              // creted with new operator
        
    devInfo = new DevInfo(node);
    delete node; node = NULL;

    if (lastErrorCode != 0) {
        LOG.info(TEXT("DevInfo NULL"));
        goto finally;
    }
       
    //
    // populate DevDetail object 
    //

    wsprintf(context, TEXT("%s/%s"), DM_ROOT_CONTEXT, DM_DEVDETAIL_CONTEXT);
    node = getManagementNode(context);              // creted with new operator
    devDetail = new DevDetail(node);
    delete node; node = NULL;
    
    if (lastErrorCode != 0) {
        //
        // Actually DevDetail is not mandatory. Reset lastErrorCode and continue...
        //
        lastErrorCode = 0;
        wsprintf(lastErrorMsg, TEXT("%s"), TEXT(""));
        //LOG.info(TEXT("DevDetail NULL"));
        //goto finally;
    }

    
    //
    // populate DevDetail/URI object 
    //

    wsprintf(context, TEXT("%s/%s"), DM_ROOT_CONTEXT, DM_DEVDETAILURI_CONTEXT);
    node = getManagementNode(context); 
    devDetail->devDetailUri = new DevDetailURI(node);
    delete node; node = NULL;

    if (lastErrorCode != 0) {
        //
        // Actually DevDetail/URI is not mandatory. Reset lastErrorCode and continue...
        //
        lastErrorCode = 0;
        wsprintf(lastErrorMsg, TEXT("%s"), TEXT(""));
        //LOG.info(TEXT("DevDetail NULL"));
        //goto finally;
    }

    syncMLDMBuilder   = new SyncMLDMBuilder();     

    syncMLDMProcessor = new SyncMLDMProcessor();   


finally:    

    return;
}

/*
 * Destructor
 */
SyncMLDMManager::~SyncMLDMManager() {

    if (syncMLDMBuilder != NULL) {
        delete syncMLDMBuilder; syncMLDMBuilder = NULL;
    }

    if (syncMLDMProcessor != NULL) {
        delete syncMLDMProcessor; syncMLDMProcessor = NULL;
    }

    if (transportAgent != NULL) {
        delete transportAgent; transportAgent = NULL;
    }
    if (syncMLDMConfig != NULL) {
        delete syncMLDMConfig; syncMLDMConfig = NULL;
    }

    if (devInfo != NULL) {
        delete devInfo; devInfo = NULL;
    }
    if (devDetail != NULL) {
        delete devDetail; devDetail = NULL;
    }
}

/*
 * Returns the root management node for the DeviceManager.
 *
 * The ManagementNode is created with the new operator and must be
 * discarded by the caller with the operator delete.
 */
ManagementNode* SyncMLDMManager::getRootManagementNode() {
    
    Win32ManagementTree managementTree = Win32ManagementTree();
    ManagementNode *ret = NULL;
    ret = managementTree.getRootManagementNode();
    return ret;
}

/*
 * Returns the management node identified by the given node pathname
 * (relative to the root management node). If the node is not found
 * NULL is returned; additional info on the error condition can be
 * retrieved calling getLastError() and getLastErrorMessage()
 *
 * The ManagementNode is created with the new operator and must be
 * discarded by the caller with the operator delete.
 */
ManagementNode* SyncMLDMManager::getManagementNode(const wchar_t* node) {
   
    Win32ManagementTree managementTree = Win32ManagementTree();
    ManagementNode *ret = NULL;
    ret = managementTree.getManagementNode(node);
    return ret;
    
}


 /*
 * Returns informations about result of DM session. It starts the DM Session
 * using a DMSessionId, that is created by client. Otherwhise if it's NULL
 * a DMSessionId is created internally.
 */
int SyncMLDMManager::startSession(wchar_t* dmSessionId, BOOL isClientNotified) {
    

    LOG.debug(TEXT("startSession"));

    wchar_t* initMsg     = NULL;
    wchar_t* responseMsg = NULL;
    wchar_t* respURI     = NULL;
    wchar_t* storeNextNonce = NULL;
    wchar_t* responseStatusMsg = NULL;

    wchar_t* syncHdrMsg  = TEXT(""); // init to empty msg
    

    
    int serverMsgId = 0; // server message ref
    int timeNumber  = 0;  
    int ret         = -1; // return result
    int cmdId       = 0; // client id message. It's has to be resetted every message.
    int msgId       = 0; // client number message

    URL url = URL(syncMLDMConfig->syncMLDMAcc->getAddr());
    Proxy proxy = Proxy();
    
    //
    // Creates the transport agent. This is cached in the instance variable
    // transportAgent so that the next phases of the sync can use the same
    // agent. Plus, the agent status is in this way preserved between the
    // multiple SyncMLDM messages.
    //
    transportAgent = new TransportAgent(url, proxy);   

    /* --------------------- auth -----------*/
    do {  
        
        safeDelete(&initMsg    );
        safeDelete(&responseMsg);

        initMsg = syncMLDMBuilder->createInitMessage(syncMLDMConfig, devInfo, dmSessionId, &cmdId, &msgId, isClientNotified, syncHdrMsg);
        if (initMsg == NULL) {        
            goto finally;
        }
    
        LOG.debug(TEXT("Initialization message:"));
        LOG.debug(initMsg);
 
        responseMsg = transportAgent->sendMessage(initMsg);
        
        if (responseMsg == NULL) {
            ret = lastErrorCode;
            LOG.debug(lastErrorMsg);        
            goto finally;
        }
        
        // reset cmdId
        cmdId       = 0;
        
        ret = syncMLDMProcessor->processInitResponse(responseMsg, &storeNextNonce);

        if (ret == -1) {
            ret = lastErrorCode;
            goto finally;
        }

        //
        // store nextNonce in Config object. If nonce is NULL auth isn't MD5
        //

        if (storeNextNonce != NULL) {
            syncMLDMConfig->syncMLDMAcc->setClientNonce(storeNextNonce);
            syncMLDMConfig->saveToDM();
        }        
        
        // serverMsgId is the MsgId of the server
        serverMsgId = syncMLDMProcessor->getServerMessageId(responseMsg);
        ret = syncMLDMProcessor->processHeader(responseMsg);
        
        syncHdrMsg = syncMLDMBuilder->createSyncHdrStatusMessage(&cmdId, serverMsgId, syncMLDMConfig, devInfo);

        respURI = syncMLDMProcessor->getRespURI(responseMsg);
        if (respURI != NULL) {
            url = respURI;
            transportAgent->setURL(url);
        }
        if ((ret < 200) || (ret > 299)) {                     
            
            // if storeNextNonce == NULL the auth isn't MD5. So don't perform authentication again..
            if (storeNextNonce == NULL) {
                //ret = ERR_AUTH_NOT_AUTHORIZED;
                break;
            
            }

        }
        else { // it's auth
            ret = 0;
            break;
        }
        timeNumber++;

    } while((timeNumber <= 1));
    
    if (ret != 0) {
        /*
        switch (ret) {
            case ERR_PROTOCOL_ERROR:
                LOG.error(TEXT("Protocol error"));
                break;

            case ERR_AUTH_NOT_AUTHORIZED:
                LOG.error(TEXT("Not authorized"));
                break;
            case ERR_AUTH_REQUIRED:
                LOG.error(TEXT("Not authorized"));
                break;

            case ERR_AUTH_EXPIRED:
                LOG.error(TEXT("Account expired; required payment"));
                break;

            case ERR_SRV_FAULT:
                LOG.error(TEXT("Server error"));
                break;

            case ERR_NOT_FOUND:                
                LOG.error(TEXT("NotFound"));
                break;
            default:    
                break;
        }
        */
        goto finally;
    }    
    
    /* ---------------------- command msg ---------------------- */
    
    // serverMsgId is the MsgId of the server
    //serverMsgId = syncMLDMProcessor->getServerMessageId(responseMsg);
    
    //
    // responseStatusMsg contains info about status adn results if exists. 
    // If responseStatusMsg is NULL then there's no action to do again
    //
    
    do {
        safeDelete(&responseStatusMsg);

        if (responseMsg == NULL) {
            //
            // Error in responseMsg
            //
            break;
        }

        responseStatusMsg = syncMLDMProcessor->processMessage(responseMsg, syncMLDMBuilder, 
                                                                &cmdId, serverMsgId, DM_ROOT_CONTEXT);

        if (responseStatusMsg == NULL) {
            //
            // No more Command
            //
            break;
        }

        safeDelete(&responseMsg);

        responseMsg = syncMLDMBuilder->createResponseStatusMessage (syncMLDMConfig, devInfo, dmSessionId, 
                                                                    &cmdId, &msgId, responseStatusMsg, syncHdrMsg);
        LOG.debug(responseMsg);

        responseMsg = transportAgent->sendMessage(responseMsg);

        // reset cmdId
        cmdId       = 0;


    } while (responseStatusMsg != NULL);
    
   

finally:
    
    safeDelete(&initMsg          );
    safeDelete(&responseMsg      );
    safeDelete(&respURI          );
    safeDelete(&storeNextNonce   );
    safeDelete(&responseStatusMsg);

    return ret;
}

  